unit ConfigU;

interface

const
  DriveLettersSubstitutionCount = 8;
var
  Configuration: record
    AddToDisksMenu: Boolean;
    DisksMenuDigit: Char;
    TapesDirectory,
    CDDrive: string;

    SituatedLocally: Boolean;

    AVComStr: string;
    RequestFolder: string;
    DriveLettersSubstitution: array[0..Pred(DriveLettersSubstitutionCount)] of string;
    AddDismount: Boolean;
  end;

function Config(ItemNumber: Integer): Boolean;

function GetRemoteDir(Path: string): string;

implementation

uses
  SysUtils,
  Registry,

  FARIntf,
  FARLang;

const
  Section = 'PAFTDR';

{
   
  (*)   
  ( )   
}

function ConfigGeneral: Boolean;
const
  TapesDirectoryHistoryID: PChar = 'PAFTDR-TapesDirectory';
  CDDriveHistoryID: PChar = 'PAFTDR-CDDrive';
var
  Radio1, Radio2: Boolean;
  DisksMenuDigitString: string;
begin
  with Configuration do
  begin
    SetLength(DisksMenuDigitString, 1);
    DisksMenuDigitString[1] := DisksMenuDigit;
    Radio1 := SituatedLocally = True;
    Radio2 := SituatedLocally <> True;
    Result := Dialog('ConfigurationGeneral', [
      Integer(DI_DOUBLEBOX),    3,  1,  72, False,  False,  DIF_NONE,  Integer(MConfigTitle),
      Integer(DI_CHECKBOX),     5,  2,   0, False,  @AddToDisksMenu,  DIF_NONE,  Integer(MConfigAddToDisksMenu),
      Integer(DI_FIXEDIT),      7,  3,   7,  True,  False,  DIF_NONE,  @DisksMenuDigitString,
      Integer(DI_TEXT),         9,  3,   0, False,  False,  DIF_NONE,  Integer(MConfigDisksMenuDigit),
      Integer(DI_TEXT),         5,  4,   0, False,  False,  DIF_BOXCOLOR or DIF_SEPARATOR,  nil,
      Integer(DI_TEXT),         5,  5,   0, False,  False,  DIF_NONE,  Integer(MConfigTapesDirectory),
      Integer(DI_EDIT),         5,  6,  70, False,  TapesDirectoryHistoryID,  DIF_HISTORY,  @TapesDirectory,
      Integer(DI_TEXT),         5,  7,   0, False,  False,  DIF_NONE,  Integer(MConfigCDDrive),
      Integer(DI_EDIT),         5,  8,  70, False,  CDDriveHistoryID,  DIF_HISTORY,  @CDDrive,
      Integer(DI_TEXT),         5,  9,   0, False,  False,  DIF_BOXCOLOR or DIF_SEPARATOR,  nil,
      Integer(DI_TEXT),         5, 10,   0, False,  False,  DIF_NONE,  Integer(MArvidIsSituated),
      Integer(DI_RADIOBUTTON),  5, 11,   0, False,  @Radio1,  DIF_GROUP,  Integer(MSituatedLocally),
      Integer(DI_RADIOBUTTON),  5, 12,   0, False,  @Radio2,  DIF_NONE,  Integer(MSituatedOnRemote),
      Integer(DI_TEXT),         5, 13,   0, False,  False,  DIF_BOXCOLOR or DIF_SEPARATOR,  nil,
      Integer(DI_BUTTON),       0, 14,   0, False,  False,  DIF_CENTERGROUP,  Integer(MOK),
      Integer(DI_BUTTON),       0, 14,   0, False,  False,  DIF_CENTERGROUP, Integer(MCancel)
      ]);

    if not Result then Exit;

    DisksMenuDigit := DisksMenuDigitString[1];
    SituatedLocally := Radio1;

    with TRegIniFile.Create('\'+Info.RootKey) do
    try
      WriteBool(Section, 'AddToDisksMenu', AddToDisksMenu);
      WriteString(Section, 'DisksMenuDigit', DisksMenuDigit);
      WriteString(Section, 'TapesDirectory', TapesDirectory);
      WriteString(Section, 'CDDrive', CDDrive);
      WriteBool(Section, 'SituatedLocally', SituatedLocally);
    finally
      Free;
    end;
  end;
end;

{
  /     (  )
     avcomstr
  [...]
}


function ConfigRequestLocal: Boolean;
const
  AVComStrHistoryID: PChar = 'PAFTDR-AVComStr';
begin
  with Configuration do
  begin
   Result := Dialog('ConfigurationRequestLocal', [
      Integer(DI_DOUBLEBOX),  3,  1,  72, False,  False,  DIF_NONE,  Integer(MRequestLocalTitle),
      Integer(DI_TEXT),       5,  2,   0, False,  False,  DIF_NONE,  Integer(MAVComStr),
   {2}Integer(DI_EDIT),       5,  3,  70,  True,  AVComStrHistoryID,  DIF_HISTORY,  @AVComStr,
      Integer(DI_TEXT),       5,  4,   0, False,  False,  DIF_BOXCOLOR or DIF_SEPARATOR,  nil,
   {4}Integer(DI_CHECKBOX),   5,  5,   0, False,  @AddDismount,  DIF_NONE,  Integer(MAddDismount),
      Integer(DI_TEXT),       5,  6,   0, False,  False,  DIF_BOXCOLOR or DIF_SEPARATOR,  nil,
      Integer(DI_BUTTON),     0,  7,   0, False,  False,  DIF_CENTERGROUP,  Integer(MOK),
      Integer(DI_BUTTON),     0,  7,   0, False,  False,  DIF_CENTERGROUP,  Integer(MCancel)
      ]);
    if not Result then Exit;

    with TRegIniFile.Create('\'+Info.RootKey) do
    try
      WriteString(Section, 'AVComStr', AVComStr);
      WriteBool(Section, 'AddDismount', AddDismount);
    finally
      Free;
    end;
  end;
end;

{
  /     (   )
    
  [...]

    
  [M:]
  [R:]
  [C:=\\ThisMachine\C]
  []
  []
  []
  []
  []
  []
  []
}

function ConfigRequestRemote: Boolean;
const
  RequestFolderHistoryID: PChar = 'PAFTDR-RequestFolder';
  DriveLettersSubstitutionHistoryID: PChar = 'PAFTDR-DriveLettersSubstitution';
var
  I: Integer;
begin
  with Configuration do
  begin
    Result := Dialog('ConfigurationRequestRemote', [
      Integer(DI_DOUBLEBOX),  3,  1,  72, False,  False,  DIF_NONE,  Integer(MRequestRemoteTitle),
      Integer(DI_TEXT),       5,  2,   0, False,  False,  DIF_NONE,  Integer(MRequestFilesFolder),
   {2}Integer(DI_EDIT),       5,  3,  70,  True,  RequestFolderHistoryID,  DIF_HISTORY,  @RequestFolder,
      Integer(DI_TEXT),       5,  4,   0, False,  False,  DIF_BOXCOLOR or DIF_SEPARATOR,  nil,
      Integer(DI_TEXT),       5,  5,   0, False,  False,  DIF_NONE,  Integer(MDriveLettersSubstitution),
   {5}Integer(DI_EDIT),       5,  6,  70, False,  DriveLettersSubstitutionHistoryID,  DIF_HISTORY,  @DriveLettersSubstitution[0],
      Integer(DI_EDIT),       5,  7,  70, False,  DriveLettersSubstitutionHistoryID,  DIF_HISTORY,  @DriveLettersSubstitution[1],
      Integer(DI_EDIT),       5,  8,  70, False,  DriveLettersSubstitutionHistoryID,  DIF_HISTORY,  @DriveLettersSubstitution[2],
      Integer(DI_EDIT),       5,  9,  70, False,  DriveLettersSubstitutionHistoryID,  DIF_HISTORY,  @DriveLettersSubstitution[3],
      Integer(DI_EDIT),       5, 10,  70, False,  DriveLettersSubstitutionHistoryID,  DIF_HISTORY,  @DriveLettersSubstitution[4],
      Integer(DI_EDIT),       5, 11,  70, False,  DriveLettersSubstitutionHistoryID,  DIF_HISTORY,  @DriveLettersSubstitution[5],
      Integer(DI_EDIT),       5, 12,  70, False,  DriveLettersSubstitutionHistoryID,  DIF_HISTORY,  @DriveLettersSubstitution[6],
      Integer(DI_EDIT),       5, 13,  70, False,  DriveLettersSubstitutionHistoryID,  DIF_HISTORY,  @DriveLettersSubstitution[7],
      Integer(DI_TEXT),       5, 14,   0, False,  False,  DIF_BOXCOLOR or DIF_SEPARATOR,  nil,
  {14}Integer(DI_CHECKBOX),   5, 15,   0, False,  @AddDismount,  DIF_NONE,  Integer(MAddDismount),
      Integer(DI_TEXT),       5, 16,   0, False,  False,  DIF_BOXCOLOR or DIF_SEPARATOR,  nil,
      Integer(DI_BUTTON),     0, 17,   0, False,  False,  DIF_CENTERGROUP,  Integer(MOK),
      Integer(DI_BUTTON),     0, 17,   0, False,  False,  DIF_CENTERGROUP,  Integer(MCancel)
    ]);
    if not Result then Exit;

    with TRegIniFile.Create('\'+Info.RootKey) do
    try
      WriteString(Section, 'RequestFolder', RequestFolder);
      WriteBool(Section, 'AddDismount', AddDismount);

      for I := 0 to Pred(DriveLettersSubstitutionCount) do
        WriteString(Section, Format('DriveLettersSubstitution%d', [I]), DriveLettersSubstitution[I]);
    finally
      Free;
    end;
  end;
end;

function Config;
begin
  Result := False; //  ,   
  case ItemNumber of
    0:
      Result := ConfigGeneral;
    1:
      if Configuration.SituatedLocally
      then Result := ConfigRequestLocal
      else Result := ConfigRequestRemote;
  end;
end;


procedure StartupInfoLoaded;
var
  I: Integer;
const
  DriveLettersSubstitutionDefaults: array[0..Pred(DriveLettersSubstitutionCount)] of string = (
    'M:', 'R:', '', '', '', '', '', '');
var
  SDisksMenuDigit: string;
begin
  with TRegIniFile.Create('\'+Info.RootKey), Configuration do
  try
    AddToDisksMenu := ReadBool(Section, 'AddToDisksMenu', True);
    SDisksMenuDigit := ReadString(Section, 'DisksMenuDigit', '4');
    if SDisksMenuDigit = ''
    then DisksMenuDigit := #0
    else DisksMenuDigit := SDisksMenuDigit[1]; // ,      NT,   '95 !
    TapesDirectory := ReadString(Section, 'TapesDirectory', 'C:\Arvid\TDR');
    CDDrive := ReadString(Section, 'CDDrive', 'E:\');

    SituatedLocally := ReadBool(Section, 'SituatedLocally', True);

    AVComStr := ReadString(Section, 'AVComStr', 'C:\Arvid\ARVID20.EXE -Y_AVCOMST.EXE');

    RequestFolder := ReadString(Section, 'RequestFolder', '\\Server\Arvid');
    for I := 0 to Pred(DriveLettersSubstitutionCount) do
    begin
      DriveLettersSubstitution[I] := ReadString(Section,
        Format('DriveLettersSubstitution%d', [I]),
        DriveLettersSubstitutionDefaults[I]);
    end;

    AddDismount := ReadBool(Section, 'AddDismount', False);
  finally
    Free;
  end;
end;


{ }

function GetRemoteDir;

  procedure Process;
  {
  C:=\\paf\c

  C:\temp   -> \\paf\c\temp
  }
  var
    I: Integer;
    DriveLetterSubstitution: string;
    P: Integer;
    Here, There: string;
  begin
    with Configuration do
      if SituatedLocally
      then
        Result := Path
      else
      begin
        Result := '';  //   

        for I := 0 to Pred(DriveLettersSubstitutionCount) do
        begin
          DriveLetterSubstitution := Configuration.DriveLettersSubstitution[I];
          if DriveLetterSubstitution <> '' then
          begin
            P := Pos('=', DriveLetterSubstitution);
            if P = 0
            then
            begin
              Here := DriveLetterSubstitution;
              There := DriveLetterSubstitution;
            end
            else
            begin
              Here := Copy(DriveLetterSubstitution, 1, Pred(P));
              There := Copy(DriveLetterSubstitution, Succ(P), $FF);  // FF :)
            end;

            if StrLIComp(PChar(Path), PChar(Here), Length(Here)) = 0 then
            begin
              Delete(Path, 1, Length(Here));
              Insert(There, Path, 1);
              Result := Path;
              Exit;
            end;
          end;
        end;
      end;
  end;

  procedure ProblemsWithF5;
  begin
    if Message(FMSG_WARNING, 'ProblemsWithF5', [
      GetMsg(MProblemsWithF5Title),
      GetMsg(MProblemsWithF5Message),
      PChar(Path),
      GetMsg(MCancel),
      GetMsg(MProblemsWithF5Button)],
      2) = 1 then
      Config(1);
  end;

begin
  Process;
  if Result = '' then
    ProblemsWithF5;
end;

initialization
  OnStartupInfoLoad := StartupInfoLoaded;
end.

