unit PluginU;

interface

uses
  Windows,
  Classes,

  FARIntf,
  LevelU;

const
  TapeDirectoryExtension = '.TDR';
type
  TPlugin = class(TObject)
  private
    Root: TLevel;
    FLevel: TLevel;
    SetFolderOpMode: Integer;
    procedure SetLevel(Value: TLevel);
  public
    CurrentPath: string;
    property Level: TLevel read FLevel write SetLevel;

    class function IsOurs(
      const Name: string;
      var Data;
      DataSize: Integer): Boolean;

    class procedure GetPluginInfo(
      var Info: TPluginInfo);

    constructor Create(const AFileName: string);
    destructor Destroy; override;

    function SetFolder(
      Dir: PChar;
      OpMode: Integer): Boolean;
    function ProcessEvent(
      Event: FE;
      Param: Pointer): Boolean; 
    function GetVirtualFindData(
      var PanelItems: PPluginPanelItemsArray;
      var ItemsNumber: Integer;
      APath: PChar): Boolean; 
  end;

implementation

uses
  SysUtils,

  Common,
  FARLang,
  ConfigU,
  RootLevelU,
  TapeLevelU;

class function TPlugin.IsOurs;
begin
  Result := TTapeLevel.IsOurs(Name, Data, DataSize);
end;

procedure TPlugin.SetLevel;
begin
  if FLevel <> Value then
  begin
    FLevel := Value;
    if SetFolderOpMode = OPM_NORMAL then
      UpdateInfoPanel(THandle(Self));
  end;
end;


var
  CDiskMenuStrings: PChar;
  CDiskMenuNumbers: Integer;
  CPluginMenuStrings: PChar;
  CPluginConfigStrings: array[0..1] of PChar;

class procedure TPlugin.GetPluginInfo;
begin
  with Info do
  begin
    StructSize := SizeOf(Info);
//    Flags: PLUGIN_FLAGS;
    CDiskMenuStrings := GetMsg(MDiskMenuString);
                                                        DiskMenuStrings := @CDiskMenuStrings;
    CDiskMenuNumbers := Ord(Configuration.DisksMenuDigit)-Ord('0');
    if CDiskMenuNumbers < 0{  } then
      CDiskMenuNumbers := 0;
                                                        DiskMenuNumbers := @CDiskMenuNumbers; //z
                                                        DiskMenuStringsNumber := Integer(Configuration.AddToDisksMenu);
    CPluginMenuStrings := GetMsg(MPluginMenuString);
                                                        PluginMenuStrings := @CPluginMenuStrings;
                                                        PluginMenuStringsNumber := 1;
    CPluginConfigStrings[0] := GetMsg(MPluginGeneralConfigString);
    CPluginConfigStrings[1] := GetMsg(MPluginRequestConfigString);
                                                        PluginConfigStrings := @CPluginConfigStrings;
                                                        PluginConfigStringsNumber := 2;
  end;
end;


constructor TPlugin.Create;
begin
  if AFileName = ''
  then Root := TRootLevel.Create(Self)
  else Level := TTapeLevel.Create(Self, AFileName);

  SetFolder('', OPM_NORMAL);  //  Level  Root
end;

destructor TPlugin.Destroy;
begin
  if Assigned(Root)
  then Root.Destroy
  else Level.Destroy;

  inherited Destroy;
end;

function TPlugin.SetFolder;
var
  RunPath, TapeName: string;
  NewPath: string;
  I: Integer;
  Index: Integer;
  TapeItem: TTapeItem;
begin
  SetFolderOpMode := OpMode;
  Result := True;

  if StrLComp(Dir, PChar(Configuration.CDDrive), Length(Configuration.CDDrive)) = 0 then
    if Assigned(Root) then
    begin
      Dir := StrRScan(Dir, '\');  // CDDrive, 
      Inc(Dir);                   //   
    end
    else
    begin
      Inc(Dir, Length(Configuration.CDDrive));
      if Dir[0]='\' then
        Inc(Dir);
    end;

  if Dir = '\' then      //      
    Inc(Dir);            // ,    Ctrl+\  

  if Dir = '..'
  then
    if CurrentPath = ''
    then
    begin
      Result := False;  //     -      '..'
      Exit;
    end
    else
    begin
      I := Length(CurrentPath);
      repeat
        Dec(I);
      until (I = 0) or (CurrentPath[I] = '\');
      NewPath := Copy(CurrentPath, 1, Pred(I));
    end
  else
  begin
    if
      (CurrentPath = '') or
      (Dir[0] = #0) or
      (Pos('\', Dir) <> 0)
    then NewPath := Dir //  (   file find)
    else NewPath := CurrentPath+'\'+Dir; // 
  end;

  NewPath := OEMUpperCase(NewPath); //     

  if Assigned(Root) //  
  then
  begin
    if NewPath = ''
    then
    begin
      CurrentPath := NewPath;
      Level := Root;
    end
    else
    begin
      RunPath := NewPath;
      TapeName := ExtractNextDir(RunPath);
      with TRootLevel(Root).Tapes do
        if Find(TapeName, Index)
        then
        begin
          TapeItem := TTapeItem(Objects[Index]);
          if not Assigned(TapeItem.TapeLevel) then
            TapeItem.TapeLevel := TTapeLevel.Create(Self,
              Configuration.TapesDirectory+'\'+TapeName+TapeDirectoryExtension);
          Level := TapeItem.TapeLevel;
        end
        else
        begin
          Result := False;
          Exit;
        end;

      Result := Level.SetFolder(PChar(RunPath), OpMode);
      if Result then
        CurrentPath := NewPath;
    end;
  end
  else //  .tdr
  begin
    Result := Level.SetFolder(PChar(NewPath), OpMode);
    if Result then
      CurrentPath := NewPath;
  end;
end;

function TPlugin.ProcessEvent;
begin
  Result := False;

  case Event of
    FE_CLOSE:
      if Assigned(Root)
      then Result := not TRootLevel(Root).CloseQuery
      else Result := not TTapeLevel(Level).CloseQuery;
  end;
end;

function TPlugin.GetVirtualFindData;
begin
  if Assigned(Root)
  then Result := TRootLevel(Root).GetVirtualFindData(PanelItems, ItemsNumber, APath)
  else Result := TTapeLevel(Level).GetVirtualFindData(PanelItems, ItemsNumber, APath);
end;

end.

