unit RootLevelU;

interface

uses
  Windows,
  Classes,

  FARIntf,
  PluginU,
  LevelU,
  TapeLevelU;

type
  TTapeItem = class(TObject)
    FindData: TWin32FindData;
    IsCD: Boolean;
    TapeLevel: TTapeLevel;
    constructor Create(AFindData: TWin32FindData);
  end;

  TRootLevel = class(TLevel)
  private
    Plugin: TPlugin;

  public
    function CloseQuery: Boolean;
  public
    Tapes: TStringList;

    constructor Create(APlugin: TPlugin);
    destructor Destroy; override;

    procedure GetOpenPluginInfo(
      var Info: TOpenPluginInfo); override;
    function GetFindData(
      var PanelItems: PPluginPanelItemsArray;
      var ItemsNumber: Integer;
      OpMode: Integer): Boolean; override;
    function ProcessKey(
      Key: Integer;
      ControlState: Integer): Boolean; override;
    function GetVirtualFindData(
      var PanelItems: PPluginPanelItemsArray;
      var ItemsNumber: Integer;
      APath: PChar): Boolean; override;
  end;

implementation

uses
  SysUtils,

  Common,
  FARLang,
  ConfigU;

// TTapeItem

constructor TTapeItem.Create;
begin
  FindData := AFindData;
  IsCD := FileIsCDDirectory(FindData.dwFileAttributes);

  //TapeLevel   ,  
end;

// TRootLevel

constructor TRootLevel.Create;
var
  SearchRec: TSearchRec;
  Error: Integer;
  TapeName: string;
begin
  Plugin := APlugin;

  Tapes := TStringList.Create;
  Tapes.Sorted := True;

  Error := FindFirst(Configuration.TapesDirectory+'\*.TDR',
    faAnyFile and not faDirectory, SearchRec);
  while Error = 0 do
  begin
    TapeName := GetTapeName(SearchRec.Name);
    with SearchRec.FindData do
    begin
      dwFileAttributes := dwFileAttributes or faDirectory;
      StrPCopy(cFileName, TapeName);
      cAlternateFileName[0] := #0;
      //     (     )
      //   ,  ~,   
    end;
    Tapes.AddObject(TapeName,
      TTapeItem.Create(TWin32FindData(Pointer(@SearchRec.FindData)^)));

    Error := FindNext(SearchRec);
  end;
  FindClose(SearchRec);
end;

destructor TRootLevel.Destroy;
var
  I: Integer;
begin
  if Assigned(Tapes) then
    with Tapes do
    begin
      for I := 0 to Pred(Count) do
        TObject(Objects[I]).Free;

      Destroy;
    end;

  inherited Destroy;
end;

//var
//  InfoPanelLinesArray: array[0..Pred(1)] of TInfoPanelLine;

var
  SFormat: string;
  STitle: string;

procedure TRootLevel.GetOpenPluginInfo;
const
  KeyBarTitles: TKeyBarTitles = (
    Titles:(nil,nil,'','','','',''{todo}{7},'',nil,nil,nil,nil);  {<x> - ,   }
    CtrlTitles:(nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil);    {nil - }
    AltTitles:(nil,nil,'','','','',nil,nil,nil,nil,nil,nil);         {'' - }
    ShiftTitles:('','',nil{3},'','','','','',nil,nil,nil,nil)
  );
begin
(*
  FillChar(InfoPanelLinesArray, SizeOf(InfoPanelLinesArray), 0);

  for I := Low(InfoPanelLinesArray) to High(InfoPanelLinesArray) do
    with InfoPanelLinesArray[I] do
      StrCopy(Data, '-');
*)
  FillChar(Info, SizeOf(Info), 0);
  with Info do
  begin
    StructSize := SizeOf(Info);
    Flags :=
      OPIF_USEFILTER           or
      OPIF_USESORTGROUPS       or
      OPIF_USEHIGHLIGHTING     or
      OPIF_ADDDOTS             or
//    OPIF_SHOWNAMESONLY       or
//    OPIF_SHOWRIGHTALIGNNAMES or
      OPIF_FINDFOLDERS         or
      OPIF_COMPAREFATTIME;

//    HostFile := PChar(FileName);

    CurDir := PChar(Plugin.CurrentPath);

    SFormat := '';  Format := PChar(SFormat);
    STitle := ' '+GetMsg(MPluginMenuString)+' ';  PanelTitle := PChar(STitle);
//    InfoLines := @InfoPanelLinesArray;
//    InfoLinesNumber := Succ(High(InfoPanelLinesArray));

//    DescrFilesArray[0] := 'descript.ion';//  ,    
//    DescrFiles := @DescrFilesArray;
//    DescrFilesNumber := Succ(High(DescrFilesArray));
//    PanelModesArray := nil; //z
//    PanelModesNumber := 0;
//    StartPanelMode := 0; // If you do not wish to change panel view mode after starting plugin, set StartPanelMode to 0.
//    StartSortMode := SM_DEFAULT;
//    StartSortOrder := 0;

    with KeyBarTitles do
    begin
      ShiftTitles[Pred(3)] := GetMsg(MKeyBarShiftF3);
    end;
    KeyBar := @KeyBarTitles;
  end;
end;

function TRootLevel.GetFindData;
var
  Size: Integer;
  I: Integer;
  TapeItem: TTapeItem;
  S: string;
begin
  Result := True;

  with Tapes do
  begin
    Size := Count*SizeOf(TPluginPanelItem);

    PanelItems := AllocMem(Size);
    ItemsNumber := Count;

    FillChar(PanelItems^, Size, 0);

    for I := 0 to Pred(Count) do
    begin
      TapeItem := TTapeItem(Objects[I]);
      with PanelItems^[I] do
      begin
        FindData := TapeItem.FindData;
  //      PackSizeHigh: DWORD;
  //      PackSize: DWORD;
  //      Flags: DWORD;     
  //      NumberOfLinks: DWORD;
        if TapeItem.IsCD
        then S := GetMsg(MCD)
        else S := GetMsg(MTape);
        Description := StrNew(PChar(S));
{
        Description := 'paf';
        Description := 'pif';
        asm
        int 3; int 3; int 3; int 3; int 3;
        end;
}        
  //      Owner: PChar;
  //      CustomColumnData: PPCharArray;
  //      CustomColumnNumber: Integer;
      end;
    end;
  end;
end;

function TRootLevel.ProcessKey;

  procedure HiddenCopyright;
  begin
    Message(FMSG_NORMAL, 'Author', [
      'PAFTDR Plugin, Version 1.4',
      'Copyright (c) 1997,98 Alexander Petrosyan (PAF), Moscow',
      'http://www.design.ru/~paf',
      'paf@design.ru; pafhome@design.ru',
      'paf@chat.ru; paf@i-connect.ru;',
      'paf@fbit.msk.su',
      GetMsg(MOK)],
      1);
  end;

begin
  Result := True;

  if inherited ProcessKey(Key, ControlState) then
    Exit;

  if Key = VK_F8 then
  begin
    HiddenCopyright;

    Exit;
  end;

  Result := False;  //  ,      ...
end;


function TRootLevel.CloseQuery;
var
  I: Integer;
begin
  with Tapes do
    for I := 0 to Pred(Count) do
      with TTapeItem(Objects[I]) do
        if Assigned(TapeLevel) then
          if not TapeLevel.CloseQuery then
          begin
            Result := False;
            Exit;
          end;

  Result := True;
end;

function TRootLevel.GetVirtualFindData;
var
  I: Integer;
begin
  Result := True;
  PanelItems := nil; //    ,      ?...
  ItemsNumber := 0;

  with Tapes do
    for I := 0 to Pred(Count) do
      with TTapeItem(Objects[I]) do
        if Assigned(TapeLevel) then
          TapeLevel.GetVirtualFindData(PanelItems, ItemsNumber, APath);
end;

end.

